package br.usp.poli.java;

/**
 * Esta interface define um jogador para o jogo de Truco.<br>
 * Um objetivo desta interface  permitir o intercmbio entre implementaes do
 * jogador (mais sobre isso ser dito na Aula 6). Para que isso funcione, 
 * fundamental que as outras classes que cuidam da lgica do Jogo no sejam
 * usadas (se forem, a sua implementao de jogador no ser intercambivel.
 * @version 1.2
 * @author FLevy
 */
public interface IJogador {
	/**
	 * Obtm o nome do Jogador.
	 * @return O nome do jogador.
	 */
	public String getNome();

	/**
	 * Analisa se deseja pedir truco.<br>
	 * Essa pergunta  feita antes de pedir para o jogador "jogar".
	 * @param valorMao O valor da mo atual.
	 * @param cartaNaMesa A carta que est na mesa (do oponente). Se o
	 * oponente ainda no jogou, a carta deve ser nula.
	 * @return Verdadeiro se deseja pedir truco ou falso caso contrrio.
	 */
	public boolean pedirTruco(int valorPartida, ICarta cartaNaMesa);

	/**
	 * Analisa um pedido de truco feito pelo outro jogador.<br>
	 * @param valorMao Quanto a mo vale com esse pedido de truco.
	 * @return Verdadeiro se o pedido foi aceito ou falso caso contrrio.
	 */
	public boolean analisarTruco(int valorMao);

	/**
	 * Representa o ato do jogador jogar uma carta em uma rodada.<br>
	 * Ele deve escolher uma de suas cartas (recebidas) e jogar.
	 * @param cartaNaMesa A carta que est na mesa, jogada pelo outro
	 * jogador ou nulo caso este jogador seja o primeiro a jogar.
	 * @return A carta jogada.
	 */
	public ICarta jogar(ICarta cartaNaMesa);

	/**
	 * Avisa ao jogador que ele ganhou (ou no) a rodada.
	 * @param ganhou Se ele ganhou (true) ou perdeu (false).
	 */
	public void avisarGanhouRodada(boolean ganhou);

	/**
	 * Avisa ao jogador que ele ganhou (ou no) a mo.
	 * @param ganhou Se ele ganhou (true) ou perdeu (false).
	 * @param pontos Quantos pontos foram disputados.
	 */
	public void avisarGanhouMao(boolean ganhou, int pontos);
}
