package br.usp.poli.java.ihc;
/*
Copyright (C) 2008 Fbio Levy Siqueira (fabiolevy@yahoo.com.br)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.InvalidPropertiesFormatException;
import java.util.Properties;

import javax.swing.ImageIcon;

import br.usp.poli.java.ICarta;

class FiguraDaCarta {
	private static final String ARQUIVO = "/cartas.properties";
	private static FiguraDaCarta singleton = null;
	private static ImageIcon traseira = null;
	private static int largura = -1, altura = -1;
	private static final String[] VALORES = {"QUATRO", "CINCO", "SEIS",
		"SETE", "DAMA", "VALETE", "REI", "AS", "DOIS", "TRES"};
	private static final String[] NAIPES = {"OUROS", "ESPADAS", "COPAS", "PAUS"};
	
	private Properties propriedades;
	
	private FiguraDaCarta() {
		propriedades = new Properties();
		try {
			propriedades.loadFromXML(FiguraDaCarta.class.getResourceAsStream(ARQUIVO));
		} catch (InvalidPropertiesFormatException e) {
			throw new ProblemaInterface("Formato incorreto do arquivo de propriedades.");
		} catch (FileNotFoundException e) {
			throw new ProblemaInterface("Arquivo de propriedades no encontrado (" + ARQUIVO + ").");
		} catch (IOException e) {
			throw new ProblemaInterface("Erro de I/O ao carregar o arquivo de configurao.");
		}
	}
	
	static ImageIcon obterCarta(ICarta carta) {
		if (singleton == null) singleton = new FiguraDaCarta();
		
		boolean ordemNaipeValor = "0".equals(singleton.propriedades.get("ordem"));
		String arquivo = "/" + singleton.propriedades.getProperty("diretorio") + "/" + singleton.propriedades.get("prefixo");
		
		if (ordemNaipeValor) arquivo += singleton.propriedades.getProperty(NAIPES[carta.getNaipe()]);
		else arquivo += singleton.propriedades.getProperty(VALORES[carta.getValor()]);
		
		arquivo += singleton.propriedades.getProperty("separador");
		
		if (ordemNaipeValor) arquivo += singleton.propriedades.getProperty(VALORES[carta.getValor()]);
		else arquivo += singleton.propriedades.getProperty(NAIPES[carta.getNaipe()]);
		
		arquivo += singleton.propriedades.getProperty("sufixo");
		arquivo += "." + singleton.propriedades.getProperty("extensao");
		
		return new ImageIcon(FiguraDaCarta.class.getResource(arquivo));
	}
	
	static ImageIcon obterTraseira() {
		if (traseira == null) {
			if (singleton == null) singleton = new FiguraDaCarta();
			
			String arquivo = "/" + singleton.propriedades.getProperty("diretorio") + "/";
			arquivo += singleton.propriedades.get("traseira");
			arquivo += "." + singleton.propriedades.getProperty("extensao");
			
			traseira = new ImageIcon(FiguraDaCarta.class.getResource(arquivo));
		}
		return traseira;
	}
	
	static int altura() {
		if (altura == -1) {
			if (singleton == null) singleton = new FiguraDaCarta();
			try {
				altura = Integer.parseInt(singleton.propriedades.getProperty("altura"));
			} catch (NumberFormatException e) {
				throw new RuntimeException("Erro ao obter as propriedades do arquivo de configurao: altura no  inteiro.");
			}
		}
		return altura;
	}
	
	static int largura() {
		if (largura == -1) {
			if (singleton == null) singleton = new FiguraDaCarta();
			largura = Integer.parseInt(singleton.propriedades.getProperty("largura"));
		}
		return largura;
	}
}
