package br.usp.poli.java.ihc;
/*
Copyright (C) 2008 Fbio Levy Siqueira (fabiolevy@yahoo.com.br)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.*;
import java.awt.Color;

import javax.swing.border.Border;
import javax.swing.text.BadLocationException;
import javax.swing.text.PlainDocument;
import javax.swing.text.AttributeSet;


@SuppressWarnings("serial")
final class TelaInicioJogo extends JDialog {
	private JComboBox comboJogador1, comboJogador2;
	private JTextField nomeJogador1, nomeJogador2;
	private Truco pai;
	private JRadioButton fixo, variavel;
	private TelaInicioJogo essaTela;
	private boolean cancelou;
	private InfoJogador[] jogadores;
	private boolean jogoManilhaFixa;
	
	private static final int LIMITE = 10;
	
	TelaInicioJogo(Truco pai) {
		super(pai, "Iniciando o jogo", true);
		this.setResizable(true);
		this.pai = pai;
		
		if (pai.getJogoManilhaFixa() == null && pai.getJogoManilhaVariavel() == null)
			throw new IllegalArgumentException("No  possvel iniciar o jogo pois no h implementaes disponveis.");
	}
	
	boolean foiCancelado() {
		return cancelou;
	}
	
	InfoJogador[] jogadores() {
		return jogadores;
	}
	
	boolean eJogoManilhaFixa() {
		return jogoManilhaFixa;
	}
	
	void escolherJogadores() {
		essaTela = this;
		this.setAlwaysOnTop(true);
		this.getContentPane().setLayout(new BorderLayout());
		
		this.add(criarPainelJogadores(), BorderLayout.PAGE_START);
		this.add(criarPainelTipoDeJogo(), BorderLayout.CENTER);
		this.add(criarPainelBotoes(), BorderLayout.PAGE_END);
		
		habilitarJogadores();
		
		this.pack();
		this.setLocationRelativeTo(super.getParent());
		this.setVisible(true);
		
		if (!cancelou) {
			jogadores = new InfoJogador[2];
			jogadores[0] = new InfoJogador((comboJogador1.getSelectedIndex() == 1), nomeJogador1.getText());
			jogadores[1] = new InfoJogador((comboJogador2.getSelectedIndex() == 1), nomeJogador2.getText());
			
			jogoManilhaFixa = fixo.isSelected();
		}	
	}
	
	private JPanel criarPainelJogadores() {
		// definindo os tipos de jogadores possveis
		
		JPanel jogadores = new JPanel(new GridBagLayout());
		jogadores.setBorder(BorderFactory.createTitledBorder("Jogadores"));
		
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		
		// Linha 0
		JLabel labelTipo = new JLabel("Tipo");
		labelTipo.setHorizontalAlignment(JLabel.CENTER);
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridx = 1;
		c.gridy = 0;
		c.insets = new Insets(5, 5, 5, 5);
		jogadores.add(labelTipo, c);
		
		JLabel labelNome = new JLabel("Nome do jogador");
		labelNome.setHorizontalAlignment(JLabel.CENTER);
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridx = 2;
		c.gridy = 0;
		c.insets = new Insets(5, 5, 2, 5);
		jogadores.add(labelNome, c);
		
		// Linha 1
		JLabel labelJogador1 = new JLabel("Jogador 1:");
		labelJogador1.setHorizontalAlignment(JLabel.RIGHT);
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridx = 0;
		c.gridy = 1;
		c.insets = new Insets(2, 5, 2, 5);
		jogadores.add(labelJogador1, c);
		
		comboJogador1 = new JComboBox();
		labelJogador1.setLabelFor(comboJogador1);
		comboJogador1.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				if (comboJogador1.getSelectedIndex() == 0) {
					nomeJogador1.setText("");
					nomeJogador1.setEnabled(true);
				} else {
					nomeJogador1.setText("Jogador 1");
					nomeJogador1.setEnabled(false);
				}
			}
					
		});
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridx = 1;
		c.gridy = 1;
		c.insets = new Insets(2, 5, 2, 5);
		jogadores.add(comboJogador1, c);
		
		nomeJogador1 = new JTextField();
		nomeJogador1.setColumns(6);
		nomeJogador1.setDocument(new PlainDocument() {
			public void insertString (int offset, String  str, AttributeSet attr) throws BadLocationException {
				if (str == null) return;
				if ((getLength() + str.length()) <= LIMITE)
					super.insertString(offset, str, attr);
			}
		});
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridx = 2;
		c.gridy = 1;
		c.insets = new Insets(2, 5, 2, 5);
		jogadores.add(nomeJogador1, c);
		
		// Linha 2
		JLabel labelJogador2 = new JLabel("Jogador 2:");
		labelJogador2.setHorizontalAlignment(JLabel.RIGHT);
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridx = 0;
		c.gridy = 2;
		c.insets = new Insets(2, 2, 2, 2);
		jogadores.add(labelJogador2, c);
		
		comboJogador2 = new JComboBox();
		labelJogador2.setLabelFor(comboJogador2);
		comboJogador2.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				if (comboJogador2.getSelectedIndex() == 0) {
					nomeJogador2.setText("");
					nomeJogador2.setEnabled(true);
				} else {
					nomeJogador2.setText("Jogador 2");
					nomeJogador2.setEnabled(false);
				}
			}
					
		});
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridx = 1;
		c.gridy = 2;
		c.insets = new Insets(2, 5, 2, 5);
		jogadores.add(comboJogador2, c);
		
		nomeJogador2 = new JTextField();
		nomeJogador2.setColumns(6);
		nomeJogador2.setDocument(new PlainDocument() {
			public void insertString (int offset, String  str, AttributeSet attr) throws BadLocationException {
				if (str == null) return;
				if ((getLength() + str.length()) <= LIMITE)
					super.insertString(offset, str, attr);
			}
		});
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridx = 2;
		c.gridy = 2;
		c.insets = new Insets(2, 5, 2, 5);
		jogadores.add(nomeJogador2, c);
		
		return jogadores;
	}
	
	private void habilitarJogadores() {
		String[] opcoes;
		int selecionadoJ1, selecionadoJ2;
		String nomeJ1 = "", nomeJ2 = "";
		
		if (comboJogador1.getSelectedIndex() == 1) {
			selecionadoJ1 = 1;
		} else {
			selecionadoJ1 = 0;
			nomeJ1 = nomeJogador1.getText();
		}
		
		if (comboJogador2.getSelectedIndex() == 1) {
			selecionadoJ2 = 1;
		} else {
			nomeJ2 = nomeJogador2.getText();
			selecionadoJ2 = 0;
		}
		
		if (fixo.isSelected()) {
			if (pai.getJogadorComputadorManilhaFixa() == null) {
				opcoes = new String[1];
				opcoes[0] = "Jogador humano";
			} else {
				opcoes = new String[2];
				opcoes[0] = "Jogador humano";
				opcoes[1] = "Computador";
			}
		} else {
			if (pai.getJogadorComputadorManilhaVariavel() == null) {
				opcoes = new String[1];
				opcoes[0] = "Jogador humano";
			} else {
				opcoes = new String[2];
				opcoes[0] = "Jogador humano";
				opcoes[1] = "Computador";
			}
		}
		
		comboJogador1.removeAllItems();
		comboJogador2.removeAllItems();
		
		for (String opcao : opcoes) {
			comboJogador1.addItem(opcao);
			comboJogador2.addItem(opcao);
		}
		
		if (selecionadoJ1 == 1 && opcoes.length > 1)
			comboJogador1.setSelectedIndex(1);
		else if (selecionadoJ1 == 0) {
			comboJogador1.setSelectedIndex(0);
			nomeJogador1.setText(nomeJ1);
		}
		
		if (selecionadoJ2 == 1 && opcoes.length > 1)
			comboJogador2.setSelectedIndex(1);
		else if (selecionadoJ2 == 0) {
			comboJogador2.setSelectedIndex(0);
			nomeJogador2.setText(nomeJ2);
		}
	}
	
	private JPanel criarPainelTipoDeJogo() {
		JPanel tipoDeJogo = new JPanel(new BorderLayout());
		tipoDeJogo.setBorder(BorderFactory.createTitledBorder("Tipo de Jogo"));
		
		fixo = new JRadioButton("Manilha fixa");
		if (pai.getJogoManilhaFixa() != null)
			fixo.setSelected(true);
		else
			fixo.setEnabled(false);
		
		fixo.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				habilitarJogadores();
			}
		});
		
		variavel = new JRadioButton("Manilha varivel");
		if (pai.getJogoManilhaVariavel() != null)
			variavel.setSelected(true);
		else
			variavel.setEnabled(false);
		
		variavel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				habilitarJogadores();
			}
		});
		
		ButtonGroup grupo = new ButtonGroup();
		grupo.add(fixo);
		grupo.add(variavel);
		
		tipoDeJogo.add(fixo, BorderLayout.PAGE_START);
		tipoDeJogo.add(variavel, BorderLayout.PAGE_END);
		
		return tipoDeJogo;
	}
	
	private JPanel criarPainelBotoes() {
		JPanel botoes = new JPanel(new FlowLayout());
		JButton ok = new JButton("Ok");
		ok.addActionListener(new ActionListener() {
			private Border normal1 = nomeJogador1.getBorder();
			private Border normal2 = nomeJogador2.getBorder();
			public void actionPerformed(ActionEvent arg0) {
				nomeJogador1.setBorder(normal1);
				nomeJogador2.setBorder(normal2);
				
				if ("".equals(nomeJogador1.getText())) {
					JOptionPane.showMessageDialog(essaTela, "O nome do jogador 1 no pode ser vazio.", "Erro", JOptionPane.ERROR_MESSAGE);
					nomeJogador1.setBorder(BorderFactory.createLineBorder(Color.red));
					nomeJogador1.grabFocus();
					return;
				}
				
				if ("".equals(nomeJogador2.getText())) {
					JOptionPane.showMessageDialog(essaTela, "O nome do jogador 2 no pode ser vazio.", "Erro", JOptionPane.ERROR_MESSAGE);
					nomeJogador2.setBorder(BorderFactory.createLineBorder(Color.red));
					nomeJogador2.grabFocus();
					return;
				}
				
				cancelou = false;
				essaTela.dispose();
			}
		});
		botoes.add(ok);
		
		JButton cancelar = new JButton("Cancelar");
		cancelar.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				cancelou = true;
				essaTela.dispose();
			}
		});
		botoes.add(cancelar);
		
		return botoes;
	}
}
